package com.dolatkia.horizontallycardslibrary

import android.animation.ObjectAnimator
import android.content.Context
import android.view.LayoutInflater
import android.view.View
import androidx.core.view.ViewCompat
import com.dolatkia.horizontallycardslibrary.databinding.ActionbarBinding

class ActionBar(context: Context, var customView: View) {

    private var translationYInAnimation: ObjectAnimator? = null
    private var translationYOutAnimation: ObjectAnimator? = null
    var actionbarBinding: ActionbarBinding =
            ActionbarBinding.inflate(LayoutInflater.from(context))

    init {
        actionbarBinding.actionBarBg.visibility = View.INVISIBLE
        actionbarBinding.actionBarBg.addView(customView)
        ViewCompat.setTranslationZ(
                actionbarBinding.closeImage,
                70f
        )
        actionbarBinding.actionBarBg.translationY = -actionbarBinding.root.height.toFloat()
    }


    fun updateUI(
            offset: Int,
            titleTopOffset: Int
    ): Boolean {
        return if (offset <= titleTopOffset) {
            startOutAnimation()
            false
        } else {
            startInAnimation()
            true
        }
    }

    private fun startOutAnimation() {
        if (actionbarBinding.actionBarBg.translationY == -actionbarBinding.root.height.toFloat()) {
            return
        }

        if (translationYOutAnimation != null && translationYOutAnimation!!.isRunning) {
            return
        }
        translationYInAnimation?.cancel()
        translationYOutAnimation =
                ObjectAnimator.ofFloat(
                        actionbarBinding.actionBarBg,
                        "translationY",
                        -actionbarBinding.root.height.toFloat()
                )
        translationYOutAnimation?.startDelay = 0
        translationYOutAnimation?.duration = 300
        translationYOutAnimation?.start()
    }

    private fun startInAnimation() {
        actionbarBinding.actionBarBg.visibility = View.VISIBLE

        if (actionbarBinding.actionBarBg.translationY == 0f) {
            return
        }

        if (translationYInAnimation != null && translationYInAnimation!!.isRunning) {
            return
        }
        if (translationYOutAnimation != null) {
            translationYOutAnimation!!.cancel()
        }
        translationYInAnimation =
                ObjectAnimator.ofFloat(actionbarBinding.actionBarBg, "translationY", 0f)
        translationYInAnimation?.startDelay = 0
        translationYInAnimation?.duration = 300
        translationYInAnimation?.start()
    }

    public fun getView(): View {
        return actionbarBinding.rootView;
    }
}